"""

metta Dummy

Dummy plugin functionality.  Various plugins that can be used as placeholders
and for testing.

"""

import logging
from typing import Any, Dict

from configerus.loaded import LOADED_KEY_ROOT

from mirantis.testing.metta.plugin import Factory, Type
from mirantis.testing.metta.environment import Environment
from mirantis.testing.metta.provisioner import METTA_PROVISIONER_CONFIG_PROVISIONER_LABEL

from .provisioner import DummyProvisionerPlugin
from .client import DummyClientPlugin
from .workload import DummyWorkloadPlugin

METTA_PLUGIN_ID_DUMMY = 'dummy'
""" All of the dummy plugins use 'dummy' as their plugin_id """


@Factory(type=Type.PROVISIONER, plugin_id=METTA_PLUGIN_ID_DUMMY)
def metta_plugin_factory_provisioner_dummy(
        environment: Environment, instance_id: str = '', fixtures: Dict[str, Dict[str, Any]] = {}):
    """ create an metta provisionersss dict plugin """
    return DummyProvisionerPlugin(environment, instance_id, fixtures)


@Factory(type=Type.CLIENT, plugin_id=METTA_PLUGIN_ID_DUMMY)
def metta_plugin_factory_client_dummy(
        environment: Environment, instance_id: str = '', fixtures: Dict[str, Dict[str, Any]] = {}):
    """ create an metta client dict plugin """
    return DummyClientPlugin(environment, instance_id, fixtures)


@Factory(type=Type.WORKLOAD, plugin_id=METTA_PLUGIN_ID_DUMMY)
def metta_plugin_factory_workload_dummy(
        environment: Environment, instance_id: str = '', fixtures: Dict[str, Dict[str, Any]] = {}):
    """ create an metta workload dict plugin """
    return DummyWorkloadPlugin(environment, instance_id, fixtures)


""" SetupTools EntryPoint METTA BootStrapping """


def bootstrap(environment: Environment):
    """ METTA_Dummy bootstrap

    We dont't take any action.  Our purpose is to run the above factory
    decorators to register our plugins.

    """
    pass



import logging
import json
from typing import Dict, Any
import inspect

from mirantis.testing.metta.plugin import Factory
from mirantis.testing.metta.cli import CliBase

logger = logging.getLogger('metta.cli.plugin')


class PluginCliPlugin(CliBase):
    """ metta Cli info plugin

    """

    def fire(self):
        """ return a dict of commands """
        return {
            'plugin': PluginCoreCli()
        }


class PluginCoreCli:
    """ Interact with the plugins """

    def hello(self, raw: bool = True):
        """ sanity test on bootstrap """
        return "Hello World, I was able to bootstrap"

    def list(self):
        """ List plugins that have been registered with the environment """

        list = {}
        for type in Factory.registry:
            list[type] = []
            for plugin_id in Factory.registry[type]:
                list[type].append(plugin_id)

        return json.dumps(list, indent=2, default=lambda X: "{}".format(X))



from mirantis.testing.metta.output import OutputBase
import logging

logger = logging.getLogger('metta.contrib.common.output.text')


class TextOutputPlugin(OutputBase):
    """ metta Output plugin a text output type

    this just gets and sets text. Nothing fancy.

    """

    def __init__(self, environment, instance_id, text: str = ''):
        """ Run the super constructor but also set class properties

        Parameters:
        -----------

        data (str) : any string data to be stored

        Raises:
        -------

        A configerus.validate.ValidationError is raised if you passed in a
        validator target and validation failed.

        An AssertionError is raised if you didn't pass in a Dict.

        """
        super(OutputBase, self).__init__(environment, instance_id)

        self.set_text(text)

    def set_text(self, data: str):
        self.text = data

    def get_output(self):
        """ retrieve assigned output


        Returns:
        --------

        The string that was assigned using .arguments()

        Raises:
        -------

        AttributeError if you tried to get text before you have assigned it
        using .arguments()

        """
        return self.text

    def info(self):
        """ Return dict data about this plugin for introspection """
        return {
            'output': {
                'text': self.text
            }
        }




from mirantis.testing.metta.client import ClientBase
from docker import DockerClient
import logging
import os

logger = logging.getLogger('metta.contrib.docker.client.dockerpy')


class DockerPyClientPlugin(ClientBase, DockerClient):
    """ metta Client plugin for docker using the docker-py library


    """

    def __init__(self, environment, instance_id, host: str, cert_path: str, tls_verify: bool = True,
                 compose_tls_version: str = 'TLSv1_2', version: str = 'auto'):
        """ Run the super constructor but also set class properties

        In order to decorate this existing class as a DockerClient, without using the
        DockerClient constructor, nor the from_env class method, we have to reproduce
        the APIClient construction.
        We could rewrite some of the DockerClient functionality, or we can just create
        a throwaway DockerClient instance and steal its ApiClient and use it.

        This also lets us easily include any other env variables that might be in
        scope.

        @Note that we don't run the docker-py constructor as we build our own client.

        Parameters:
        -----------

        All of the arguments are handed off to the DockerClient.from_env class method.
        All arguments are converted to the related ENV equivalent if it had been set
        outside of this python code.

        host (str) [DOCKER_HOST] what daemon host to use (docker socket)
        cert_path (str) [DOCKER_CERT_PATH] path to rsa keys for authorization
        tls_verify (bool) [DOCKER_TLS_VERIFY] should the client pursue TLS verification
        compose_tls_version (str) [COMPOSE_TLS_VERSION] what TLS version should
            the Docker client use for docker compose.


        """
        super(ClientBase, self).__init__(environment, instance_id)

        logger.debug(
            "Configuring docker client with args for host:{}".format(host))

        self.host = host
        self.cert_path = cert_path
        self.tls_verify = '1' if tls_verify else '0'
        self.compose_tls_version = compose_tls_version

        env = os.environ.copy()
        env['DOCKER_HOST'] = self.host
        env['DOCKER_CERT_PATH'] = self.cert_path
        env['DOCKER_TLS_VERIFY'] = self.tls_verify
        env['COMPOSE_TLS_VERSION'] = self.compose_tls_version

        throwaway = DockerClient.from_env(environment=env, version=version)
        self.api = throwaway.api

    def info(self):
        """ Return dict data about this plugin for introspection """
        return {
            'docker': {
                'host': self.host,
                'cert_path': self.cert_path,
                'tls_verify': self.tls_verify,
                'compose_tls_version': self.compose_tls_version
            }
        }
